//PRINT ONLY TYPE NOT CONFIG
#include <stdbool.h>
#include <math.h> 
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

/// The Simulation Box is a CUBE with periodic boundary conditions
#define L 1         // Number of particles per Box Length (side)
#define lambda 0.0729931*400
//#define lambda3 lambda*lambda*lambda

#define MCSTEPS 10000000000   // Total # of Monte Carlo Steps
#define WRITE_MASTER  10000   // Every WRITE writes program output
#define MBIG 1000000000    
#define MSEED 161803398
#define MZ 0
#define FAC (1.0/MBIG)

double lambda3 = lambda*lambda*lambda;
int WRITE;

typedef struct{
  double x,y,z;
} vec3D;
 
typedef struct{
  double sig,sig2; // sig1 is the diameter of the particles 
  double side,hside,hsig,Volume,Volume0;        // side length of the box 
  double rv,rv2,Vgap,Vgap2;       // verlet radius etc...     
  double eps_mc;            // Monte Carlo displacement parameter
  double cut,cut1;

  double chap_diameter;
} SYSTEM;

typedef struct{
  double x,y,z;      /* Coordinates (x,y,z)         */
  double Vx,Vy,Vz,V2x,V2y,V2z;   /* Verlet Coordinates (x,y,z)  */
  double ex,ey,ez;   /* directions of SC vectors  */
  double px,py,pz;   /* directions of patch vectors  */
  int verl[3000];    //  verlet list vectors
  int Nverl;        // # of particles in the verlet list
  int type,flag,adsflag,chapflag;
  double eng;
} PARTICLE;


SYSTEM S;
PARTICLE *Elem;


void set_all(void);
void set_Elements(void);
void set_Verlet();

int check_overlap(int); 
void set_Ns(void);

double Interaction_1p(int);
double Interaction_1p2(int);
void MC_xyz(int);
void MC_rot(int);
void MC_rot_p(int);
void read_Elements(void);
void makemovie();
void event(int);
int write_track4(int);
void write_exchanges(int, int);
double overlap(int,int,double,double);
double sameatt(int,int);
double diffatt(int,int);
double micelle(int,int,double);
double mixedatt(int,int,double);
double wallattraction(int,int,double);
double patch(int,int);
void swap02(int,int);
void swapto1(int,int);
void swap12(int,int);
void swap10(int,int);
void Exchange(int, int);
void Exchange1(int);
void Scale(void);

void Rotate(int,double, vec3D);
void Rotate_p2(int,double);
vec3D axisRND(void);
double VectorProductAbs(vec3D,vec3D);
long Xseed=-4;
double ran3(long *);

FILE *out,*moviefile,*in, *eventfile, *eventtrackfile;
FILE *chaperone_file, *bindingfile, *exchangefile;

double sigma0,length,hlength,blength,tlength,htlength,hblength;//epsilonpatch,epsilonbody,epsiloncaps
double phi,delta,delta1,ang1,eng0,eng1,eng2
       ,epsilonmic0,epsilonmic1,epsilonmic01,epsilonmix0,epsilonmix1
       ,epsilonpatch2,epsilonpatch1,scaleside,epsilonwall0,epsilonwall1;
double mu, deltaMu_ivbeta;
int frame=0;
int freq02,freq01,freq21; //freq of flipping
int N,N0,NEX,Ns, Nslist[10000],Nspos[10000];

void   calculate_binding_energies(int);
double system_potential_energy(void);
double system_potential_energy_inverse(void);
int Nchap;
double Nchap_double;
double Nchap_ratio;
double eps_chaperone_fibril_ratio;
double epsilon_chaperone_fibril;
bool trigger_erase_monomers = false;
bool equilibrium_coverage_simulation = false;
bool rate_simulation = false;

double micelle_average = 0.0, micelle_variance = 0.0,short_average = 0.0; 
double short_variance = 0.0, wall_average = 0.0, wall_variance = 0.0;
int kaunt = 0, kaunt_freq = 100;

int chap_miss = 0;
int exchange_counter = 0;       
int add_counter = 0, erase_counter = 0;
bool setting_chaperones = true;

double potential_energy_before=0., potential_energy_after=0., initial_potential_energy=0.; 
double deltaE_cumulative_over_WRITE=0., deltaE_cumulative_over_simulation=0.;
double razlika_energij = 0.;
double after_min_before = 0.;
double after_min_before_cumulative = 0.;
// InVERSE ----------______________--------________---____---____--____---____--_____________-
double potential_energy_before_inverse=0., potential_energy_after_inverse=0., initial_potential_energy_inverse=0.; 
double razlika_energij_inverse = 0.;
double after_min_before_inverse = 0.;
double after_min_before_cumulative_inverse = 0.;

bool SPCError_on = false, SPCError_off = true;
int chapError = 0;
int SPCError = 0;
int stevilo_beta;
double soluble_volume_fraction, chap_volume_fraction;
double soluble0_volume, chap0_volume;
double chap_length,hchap_length, tchap_length, htchap_length;
double mu_excess;
bool skaliranje = true;
int fak = 0;

double temperature = 1.0;
int beta_limit = 6;
/**********************************************************
			  MAIN
**********************************************************/
void main(){

  double flipacoin,dum;
  int oldtype,i,j,dum01,dum02,dum1,dum21,ran;
  srand48(123456);
  int nseed; 
  int counterWRITE = 1;
  int check1,check2;

  WRITE = WRITE_MASTER;
  in=fopen("input_local.in","r");
  check1 = fscanf(in,"%lf %lf %lf %lf %lf %lf %lf %lf %d %d %lf %lf %lf %lf %lf %lf %d %lf %lf %d %lf\n"
                  ,&length,&chap_length,&sigma0,&epsilonmic0,&epsilonmic1,&eng1,&eng2,&S.cut
                  ,&freq02,&freq01,&phi,&epsilonwall0,&delta,&delta1,&epsilonwall1,&epsilonmic01
                  ,&N0,&epsilonpatch1,&epsilonpatch2,&nseed,&mu);
  check2 = fscanf(in,"%lf %lf %lf\n", &Nchap_ratio, &eps_chaperone_fibril_ratio, &temperature);
  assert(check1 == 21);
  assert(check2 == 3);
  printf("-------------------------------------------------------\n");
  printf("mon length               %.2lf\nchap length              %.2lf\nsigma                    %.2lf\nmicelle s-s              %.2lfkT\nmicelle i-i              %.2lfkT\nmu s->i, i->beta         %.2lfkT\nmu s->beta               %.2lfkT\ncutoff length            %.2lf\nswap s->beta delay       %d\n swap s->i, i->beta delay        %d\n? %.2lf\ns-f interakcija                   %.2lfkT\n(s-beta - s-s)                    %.2lfkT\n(i-beta - i-i)                   %.2lfkT\ni-f interakcija	           	  %.2lfkT\ns-i interakcija		             %.2lfkT\nN0 %d\nbeta-beta interakcija		       %.2lfkT\nf-f interakcija	    %.2lfkT\nrandom seed			 	 %d\n monomer chemical potential in solution		 %.2lf\nchap concentration / mon concentration 			%.2lf\nchap binding energy / mon binding energy (to fibril)       	 %.2lf\ntemperature		 			%.2lf \n",length,chap_length,sigma0,epsilonmic0,epsilonmic1,eng1,eng2,S.cut,freq02,freq01,phi,epsilonwall0,delta,delta1,epsilonwall1,epsilonmic01,N0,epsilonpatch1,epsilonpatch2,nseed,mu,Nchap_ratio, eps_chaperone_fibril_ratio, temperature);
  deltaMu_ivbeta = eng2 - eng1; 	// zaobjame tudi simetricni default primer eng1 = 10, eng2 = 20 	
  printf("-------------------------------------------------------\n");

  if (freq02 <= 5000){
    rate_simulation = true;
    WRITE = WRITE_MASTER;
  }
  else{
    equilibrium_coverage_simulation = true; 
    WRITE = WRITE_MASTER * 10;
  }
  printf("WRITE = %d \n", WRITE);

  if (mu <= 5.4) NEX=600;
  else if (mu <= 5.6) NEX=730;
  else if (mu <= 5.9) NEX=890;
  else if (mu <= 6.1) NEX=1100;
  else if (mu <= 6.3) NEX=1330;
  else if (mu <= 6.5) NEX=1620;
  else if (mu <= 6.7) NEX=1970;
  else if (mu <= 6.9) NEX=2400;
  else NEX=3000;			// ce NEX=3000 dobim bug, nevem zakaj
  if (mu >= 4.5) beta_limit = 40;
  else if (mu >= 3.7) beta_limit = 30;
  else if (mu >= 1.0) beta_limit = 12;
  Ns=0;
  if (Nchap_ratio < 0.0){
    Nchap_ratio = -Nchap_ratio;
    trigger_erase_monomers = true;
  }
  Nchap_double = NEX * Nchap_ratio;
  Nchap = (int)(Nchap_double);
  N=N0+NEX + Nchap;
  printf("N = %d, NEX = %d, Nchap = %d, N0 = %d, NEX+Nchap+N0 = %d \n\n", N,NEX,Nchap,N0,NEX+Nchap+N0);
  epsilon_chaperone_fibril = epsilonwall0 * eps_chaperone_fibril_ratio;

  scaleside=150.0;
  printf("lalalal %lf\n",scaleside);

  eng0=0.0;
  ang1=0.0;
  freq21=freq01;
  epsilonmix0=epsilonmic0+delta;
  epsilonmix1=epsilonmic1+delta1;
  hlength=0.5*length;
  blength=0.7*length;
  hblength=0.5*blength;
  tlength=0.2*length;
  htlength=0.5*tlength;
  if (chap_length < length){
    tchap_length = 0.2 * chap_length; // makes spherical chaperones possible
  }
  else{
    tchap_length = tlength;
  }
  hchap_length = 0.5 * chap_length;
  htchap_length = 0.5 * tchap_length;
  soluble0_volume = length * M_PI*0.5*0.5 + 4.0*M_PI*0.5*0.5*0.5 / 3.0;
  chap0_volume =    chap_length * M_PI*0.5*0.5 + 4.0*M_PI*0.5*0.5*0.5 / 3.0;

  printf("Pred set_all: %d  %d  %d  \n",N, NEX, Nchap);
  set_all();
  printf("Po set_all: %d  %d  %d  \n",N, NEX, Nchap);
  printf("EVO\n");

  int ii,check, Nprej;
  for (i=1;i<=nseed;i++) dum=drand48();

  if ((equilibrium_coverage_simulation) && !(rate_simulation)){
    printf("Ns je %d\n",Ns);
    for (i=1;i<=60;i++){
      if (i > 6) makemovie(-i);
      stevilo_beta = write_track4(-i*WRITE);
      printf("----------------------------------------------_____--------");
      printf("nastavljanje simulacijske skatle %d \n %lf ---- \n", i, mu);
      printf("Ns:   %d Nsol: %d ||S.side:   %lf ||svasta:   %lf\n",Ns, N-Nchap-N0,S.side,(N*2.87)/(S.side*S.side*S.side));
      Nprej = N;
      for (j=N0+Nchap+1;j<=Nprej;j++){
        int drek = (int)((double)(N-N0-Nchap)*drand48()+1+N0+Nchap);
        Exchange1(drek);
      }
      printf("Po %d Exchangih N = %d\n", Nprej-Nchap-N0, N); 
      if ((mu<=5.4) && (i%3==0)){
        S.side=cbrt(((double)(NEX)/(double)(N-N0-Nchap)))*S.side;
        S.hside=0.5*S.side;
        S.Volume=S.side*S.side*S.side;
        S.Volume0=S.side*S.side*S.side - S.side*12.0*12.0;
        S.hside=S.side*0.5;
        Scale();
        }
      else if (mu>5.4){
        S.side=cbrt(((double)(NEX)/(double)(N-N0-Nchap)))*S.side;
        S.hside=0.5*S.side;
        S.Volume=S.side*S.side*S.side;
        S.Volume0=S.side*S.side*S.side - S.side*12.0*12.0;
        S.hside=S.side*0.5;
        Scale();
        }
      out=fopen("track3","a");
      fprintf(out,"%d %d %lf %lf\n",i,N-N0-Nchap-NEX,S.side,(N*2.87)/(S.side*S.side*S.side));
      fclose(out);
    }
    soluble_volume_fraction = (double)(NEX)/S.Volume0 * soluble0_volume;
    chap_volume_fraction = (double)(Nchap)/S.Volume0 * chap0_volume;
    mu_excess = log(1 - soluble_volume_fraction - chap_volume_fraction);
    printf("sol volume fraction>> %lf    chap volume fraction>> %lf  %lf \n", soluble_volume_fraction,chap_volume_fraction, mu_excess);
       // ________-----------------_______________----
    if (trigger_erase_monomers){
      lambda3 = 1000*lambda3;
    }
    mu = mu - mu_excess;
    printf("mu novi je>> %lf \n", mu);
    makemovie(-100);
    stevilo_beta = write_track4(-100*WRITE);
    makemovie(-200);
    stevilo_beta = write_track4(-200*WRITE);
    set_Ns();
    for (i=1;i<=10;i++){
      for (j=1;j<=Ns;j++){
        Exchange(0, (int)((double)(Ns)*drand48()+1));
      } 
      makemovie(-i*10);
      stevilo_beta = write_track4(-i*WRITE*10);
    }
       //_________---------_________-----______--____--___
    makemovie(-300);
    stevilo_beta = write_track4(-300*WRITE);
    printf("\n\n stevilo out of box chaperonov::   %d \n\n",chap_miss);

    exchange_counter = 0;
    add_counter = 0;
    erase_counter = 0;
    skaliranje = false;
  }
  if ((rate_simulation) && !(equilibrium_coverage_simulation)){

    set_Ns();

    soluble_volume_fraction = (double)(NEX)/S.Volume0 * soluble0_volume;
    chap_volume_fraction = (double)(Nchap)/S.Volume0 * chap0_volume;
    mu_excess = log(1 - soluble_volume_fraction - chap_volume_fraction);
    printf("sol volume fraction>> %lf    chap volume fraction>> %lf  %lf \n", soluble_volume_fraction,chap_volume_fraction, mu_excess);
    mu = mu - mu_excess;
    printf("mu novi je>> %lf \n", mu);

    printf("Ns pri rate_simulations pred Exchangih = %d\n", Ns);
    for (i=1;i<=20;i++){
      for (j=1;j<=Ns;j++){
        Exchange(0, (int)((double)(Ns)*drand48()+1));
      } 
      makemovie(0);
      stevilo_beta = write_track4(0);
    }
    printf("Ns pri rate_simulations pooo Exchangih = %d\n", Ns);
    skaliranje = false;
  }


  initial_potential_energy = system_potential_energy();  
  initial_potential_energy_inverse = system_potential_energy_inverse();  
  printf("\nzacetna potencialna energija je %lf \n\n", initial_potential_energy);
  potential_energy_before = initial_potential_energy;
  potential_energy_before_inverse = initial_potential_energy_inverse;

  for (i=1;i<=MCSTEPS;i++){ // GLAVNI LOOP

    frame=i;
    for (j=1;j<=Ns;j++){
      ii=Nslist[j];
      assert(Elem[ii].type == 0);
      if ((fabs(Elem[ii].y)<6.0) && (fabs(Elem[ii].z)<6.0)){
        printf("set something's wrong %d %d %d %lf %lf\n",frame,ii,Elem[ii].flag,Elem[ii].y,Elem[ii].z);
	goto konec_simulacije;
        }
      }

    ran=(int)((double)(30000)*drand48()+1);
    check=(int)((double)(30000)*drand48()+1);

    if (ran!=check){
      for (j=1;j<=N;j++) MC_xyz((int)((double)(N)*drand48()+1));
      for (j=1;j<=N;j++) MC_rot((int)((double)(N)*drand48()+1));
    }
    else{
      for (j=1;j<=Ns;j++) {
        Exchange(i, (int)((double)(Ns)*drand48()+1));
        }
      }
    if ((rate_simulation) && (i > 100*WRITE)){
      for (j=N0+Nchap+1;j<=N;j++) {
        oldtype=Elem[j].type;
        if (oldtype==0) {
          flipacoin=drand48();
            if (flipacoin<0.5) {
              dum01=(int)(drand48()*(freq01+1));
              check=(int)(drand48()*(freq01+1));
              if ((dum01==check) && (Elem[j].adsflag==1)) swapto1(j,i);
            }
          if (flipacoin>=0.5) {
            dum02=(int)(drand48()*(freq02+1));
            check=(int)(drand48()*(freq02+1));
        //    if ((dum02==check) && (Elem[j].adsflag == 1))  swap02(j);
            if (dum02==check)  swap02(j,i);
          }
        }
          if (oldtype==1) {
            flipacoin=drand48();
            if (flipacoin<0.5) {
              dum1=(int)(drand48()*(freq01+1));
              check=(int)(drand48()*(freq01+1));
              if (dum1==check)  swap10(j,i);
            }
            if (flipacoin>=0.5) {
              dum1=(int)(drand48()*(freq21+1));
              check=(int)(drand48()*(freq21+1));
              if (dum1==check)  swap12(j,i);
            }
          }
        if (oldtype==2) {
          flipacoin=drand48();
            if (flipacoin<0.5) {
              dum21=(int)(drand48()*(freq21+1));
              check=(int)(drand48()*(freq21+1));
              if (dum21==check) swapto1(j,i);
            }	
          if (flipacoin>=0.5) {
            dum02=(int)(drand48()*(freq02+1));
            check=(int)(drand48()*(freq02+1));
            if (dum02==check)  swap02(j,i);
          }
        }
//      stevilo_beta = write_track4(j);
      }
    }
    if (((i%(WRITE)==0))){
      calculate_binding_energies(i);
      stevilo_beta = write_track4(i);
      if (i % (WRITE*10) == 0){

      makemovie(i/WRITE); 

      potential_energy_after = system_potential_energy();
      after_min_before = potential_energy_after - potential_energy_before;
      razlika_energij = after_min_before - deltaE_cumulative_over_WRITE; 
      after_min_before_cumulative += after_min_before; 


      printf("Energijska bilanca>>  %d  %lf  %lf  %lf  %lf  %lf", i, razlika_energij, after_min_before,
      deltaE_cumulative_over_WRITE, after_min_before_cumulative, deltaE_cumulative_over_simulation);
      potential_energy_before = potential_energy_after;
      // ---------___----____--_____--____--____---___---___--____--INVERSE locim od DIRECT
      printf("      INVERSE      \n");
      potential_energy_after_inverse = system_potential_energy_inverse();
      after_min_before_inverse = potential_energy_after_inverse - potential_energy_before_inverse;
      razlika_energij_inverse = after_min_before_inverse - deltaE_cumulative_over_WRITE; 
      after_min_before_cumulative_inverse += after_min_before_inverse; 

      printf("Energijska bilanca>>  %d  %lf  %lf  %lf  %lf  %lf \n\n",
             i, razlika_energij_inverse, after_min_before_inverse, deltaE_cumulative_over_WRITE,
             after_min_before_cumulative_inverse, deltaE_cumulative_over_simulation);
      potential_energy_before_inverse = potential_energy_after_inverse;
      deltaE_cumulative_over_WRITE = 0.;

      exchange_counter = 0;
      add_counter = 0;
      erase_counter = 0;
      counterWRITE += 1;
      }
    }
    if (stevilo_beta >= beta_limit) goto konec_simulacije;
    if (fak == 1) goto konec_simulacije;
  }
konec_simulacije:
  makemovie(i/WRITE);
  printf("\n\nDosezeno stevilo beta delcev 12. Predhodno zakljucena simulacija:::\n\n");
} //close main

/**********************************************************
			SET_ALL
**********************************************************/

void set_all(void){

  S.side=scaleside*(double)(L);
  S.Volume=S.side*S.side*S.side;
  S.Volume0=S.side*S.side*S.side - S.side*12.0*12.0;
  S.hside=S.side*0.5; 
  if (equilibrium_coverage_simulation){
    out=fopen("track3","a");
    fprintf(out,"side volume %lf %lf\n",S.side,S.Volume);
    fclose(out);
  }

  S.sig=sigma0;
  S.sig2=S.sig*S.sig;  
  S.hsig=0.5*S.sig;
  if (length >= chap_length){
    S.rv=1.6*(length + S.cut);  // verlet radius  
    S.Vgap=.5*(S.rv-(length + S.cut)); 
  }
  else{
    S.rv=1.6*(chap_length + S.cut);  // verlet radius  
    S.Vgap=.5*(S.rv-(chap_length + S.cut)); 
  }
  S.rv2=S.rv*S.rv;
  S.Vgap2=S.Vgap*S.Vgap;
  S.eps_mc=0.03;    
  
  moviefile=fopen("moviewall33","w");
  fclose(moviefile);   // This erases previous files with this name!!!
  eventfile=fopen("eventmovie","w");
  fclose(eventfile);   // This erases previous files with this name!!!
  eventtrackfile=fopen("eventtrack","w");
  fclose(eventtrackfile);   // This erases previous files with this name!!!

  out = fopen("track4", "w");
  fclose(out); 
  bindingfile = fopen("binding_energies.dat", "w");
  fclose(bindingfile);
  exchangefile = fopen("exchanges.dat", "w");
  fclose(exchangefile);

  if (equilibrium_coverage_simulation){
    Elem=(PARTICLE *) malloc((100*N+1)*sizeof(PARTICLE));  
    out = fopen("track3", "w");
    fclose(out);  
    set_Elements(); 
  }
  else if (rate_simulation) read_Elements();
  set_Verlet();  
}
/*-----------------------------------------------------*/
void set_Elements(void){

  int k,j,c;
  double d2,theta,z,abs;
  read_Elements();

for (j=1;j<=N;j++) (Elem[j].flag=0);

for (k=N0+1;k<=N0+Nchap;k++){
  if (Nchap == 0) printf("posran drek\n");
  Elem[k].eng = 0.0;
  Elem[k].type = 9;

  again_chap:
  Elem[k].x=(.5-drand48())*(S.side);
  Elem[k].y=(.5-drand48())*(S.side);
  Elem[k].z=(.5-drand48())*(S.side);

  theta=drand48()*2*M_PI;
  z=(.5-drand48())*2;
  Elem[k].ex=sqrt(1.0-z*z)*cos(theta);
  Elem[k].ey=sqrt(1.0-z*z)*sin(theta);
  Elem[k].ez=z;

  for (j=1;j<k;j++){
    d2=overlap(k,j,hlength,hlength);
    if (d2<S.sig){
    printf("chaperone again    %lf  <   %lf\n", d2, S.sig);      
    goto again_chap;
    }
  }
  Elem[k].px=(.5-drand48())*2;
  Elem[k].py=(.5-drand48())*2;
  Elem[k].pz=(-Elem[k].ex*Elem[k].px-Elem[k].ey*Elem[k].py)/Elem[k].ez;
  abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
  Elem[k].px=Elem[k].px/abs;
  Elem[k].py=Elem[k].py/abs;
  Elem[k].pz=Elem[k].pz/abs;
  abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
  Elem[k].chapflag = 0;
  Elem[k].adsflag = 0;
  Elem[k].flag = 0; 
  }
for (k=N0 + Nchap + 1;k<=N;k++){
  Elem[k].eng = 0.0;
  Elem[k].type = 0;

  again:
  Elem[k].x=(.5-drand48())*(S.side);
  Elem[k].y=(.5-drand48())*(S.side);
  Elem[k].z=(.5-drand48())*(S.side);

  theta=drand48()*2*M_PI;
  z=(.5-drand48())*2;
  Elem[k].ex=sqrt(1.0-z*z)*cos(theta);
  Elem[k].ey=sqrt(1.0-z*z)*sin(theta);
  Elem[k].ez=z;

  for (j=1;j<k;j++){
    d2=overlap(k,j,hlength,hlength);
    if (d2<S.sig){
      printf("soluble again    %lf  <   %lf\n", d2, S.sig);      
      goto again;
    }
  }
  Elem[k].px=(.5-drand48())*2;
  Elem[k].py=(.5-drand48())*2;
  Elem[k].pz=(-Elem[k].ex*Elem[k].px-Elem[k].ey*Elem[k].py)/Elem[k].ez;
  abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
  Elem[k].px=Elem[k].px/abs;
  Elem[k].py=Elem[k].py/abs;
  Elem[k].pz=Elem[k].pz/abs;
  abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
  Elem[k].chapflag = 0;
  Elem[k].adsflag = 0;
  Elem[k].flag = 0; 
  }
}
/*-------------------____________----_____---____---___-------------*/
void set_Ns(){
  int i;
  for (i=N0+Nchap+1;i<=N;i++){
    Nslist[i]=0;
    Nspos[i]=0;
  }
  //printf("N is %d and N0 is %d\n",N,N0);
  for (i=N0+Nchap+1;i<=N;i++){
    if ((fabs(Elem[i].y)>=6.0) || (fabs(Elem[i].z)>=6.0)) {
      Ns++;
      Elem[i].flag=1;
      Nslist[Ns]=i;
      Nspos[i]=Ns;
    }
  }
  printf("Ns is %d\n",Ns);
}
/*-----------------------------*/
void set_Verlet(){
  int i,k;
  double r2;
  double dx,dy,dz;

  for (i=1;i<=N;i++){
    Elem[i].Nverl=0;
    Elem[i].Vx=Elem[i].x;
    Elem[i].Vy=Elem[i].y;
    Elem[i].Vz=Elem[i].z;
    }

  for (i=1;i<N;i++){
    for (k=i+1;k<=N;k++){

    dx=Elem[i].x-Elem[k].x;
    dy=Elem[i].y-Elem[k].y;
    dz=Elem[i].z-Elem[k].z;

    if (dx>=S.hside) dx-=S.side;
    if (dy>=S.hside) dy-=S.side;
    if (dz>=S.hside) dz-=S.side;

    if (dx<-S.hside) dx+=S.side;
    if (dy<-S.hside) dy+=S.side;
    if (dz<-S.hside) dz+=S.side;

    r2=(dx*dx+dy*dy+dz*dz);
    if (r2<(S.rv2)){ 
      Elem[i].Nverl++; 
      Elem[i].verl[Elem[i].Nverl]=k;
      Elem[k].Nverl++; 
      Elem[k].verl[Elem[k].Nverl]=i;	
      } 
    }   
  }
}
/**********************************************************
			   MONTE CARLO
**********************************************************/
void MC_xyz(int k){

  double dx,dy,dz;
  double oldX,oldY,oldZ;
  double r2;
  double en_old = 0.0,en_new = 0.0;
  int oldadsflag,j,oldflag, oldchapflag;
  double deltaEnergy = 0.;

  if (Elem[k].type == 3) return;

  //check verlet for both ends
  dx=Elem[k].x-Elem[k].Vx;
  dy=Elem[k].y-Elem[k].Vy;
  dz=Elem[k].z-Elem[k].Vz;

  if (dx>=S.hside) dx-=S.side;
  if (dy>=S.hside) dy-=S.side;
  if (dz>=S.hside) dz-=S.side;
  if (dx<-S.hside) dx+=S.side;
  if (dy<-S.hside) dy+=S.side;
  if (dz<-S.hside) dz+=S.side;

  r2=(dx*dx+ dy*dy+ dz*dz);
  if (r2>S.Vgap2){
    set_Verlet();
   } 
  oldX=Elem[k].x;
  oldY=Elem[k].y;
  oldZ=Elem[k].z;

  en_old=Interaction_1p(k);   

  Elem[k].flag=0;
  if((Elem[k].type==0) && ((fabs(Elem[k].y)>=6.0) || (fabs(Elem[k].z)>=6.0))) Elem[k].flag=1;

  oldflag=Elem[k].flag;
  oldadsflag=Elem[k].adsflag;
  oldchapflag = Elem[k].chapflag;

  Elem[k].x+=S.eps_mc*(0.5-drand48());
  Elem[k].y+=S.eps_mc*(0.5-drand48());
  Elem[k].z+=S.eps_mc*(0.5-drand48());

  if (Elem[k].x>=S.hside) Elem[k].x-=S.side;
  if (Elem[k].y>=S.hside) Elem[k].y-=S.side;
  if (Elem[k].z>=S.hside) Elem[k].z-=S.side;
  if (Elem[k].x<-S.hside) Elem[k].x+=S.side;  
  if (Elem[k].y<-S.hside) Elem[k].y+=S.side; 
  if (Elem[k].z<-S.hside) Elem[k].z+=S.side;

  if (Elem[k].type==3){ // reject if the fibril gets out of borders
    if ((fabs(Elem[k].y)>4.0) || (fabs(Elem[k].z)>4.0)){
      Elem[k].x=oldX;
      Elem[k].y=oldY;
      Elem[k].z=oldZ;
      Elem[k].flag=oldflag;
      Elem[k].adsflag=oldadsflag;
      Elem[k].chapflag = oldchapflag;

  if (Elem[k].x>=S.hside) Elem[k].x-=S.side;
  if (Elem[k].y>=S.hside) Elem[k].y-=S.side;
  if (Elem[k].z>=S.hside) Elem[k].z-=S.side;
  if (Elem[k].x<-S.hside) Elem[k].x+=S.side;  
  if (Elem[k].y<-S.hside) Elem[k].y+=S.side; 
  if (Elem[k].z<-S.hside) Elem[k].z+=S.side;
//set_Verlet();
      return;
    }
  }
  dx=Elem[k].x-Elem[k].Vx;
  dy=Elem[k].y-Elem[k].Vy;
  dz=Elem[k].z-Elem[k].Vz;
  if (dx>=S.hside) dx-=S.side;
  if (dy>=S.hside) dy-=S.side;
  if (dz>=S.hside) dz-=S.side;
  if (dx<-S.hside) dx+=S.side;
  if (dy<-S.hside) dy+=S.side;
  if (dz<-S.hside) dz+=S.side;

  r2=(dx*dx+ dy*dy+ dz*dz);
  if (r2>S.Vgap2){
    set_Verlet();
  } 
  // safety for old overlaps. You can make them go away if you accept the moves even though
  // a new position will still result in an overlap
  if ((int)(en_old)==999){
    printf("\n\nOverlap v premiku delca %d (en_old = 999) %lf %lf %lf \n",
                                      k, Elem[k].x, Elem[k].y, Elem[k].z);
    fak = 1;
   // goto xyz_Ns;
  }

  en_new=Interaction_1p(k);
  if ((int)(en_new)==999){ // reject
    Elem[k].x=oldX;
    Elem[k].y=oldY;
    Elem[k].z=oldZ;
    Elem[k].flag=oldflag;
    Elem[k].adsflag=oldadsflag;
    Elem[k].chapflag = oldchapflag;

    if (Elem[k].x>=S.hside) Elem[k].x-=S.side;
    if (Elem[k].y>=S.hside) Elem[k].y-=S.side;
    if (Elem[k].z>=S.hside) Elem[k].z-=S.side;
    if (Elem[k].x<-S.hside) Elem[k].x+=S.side;  
    if (Elem[k].y<-S.hside) Elem[k].y+=S.side; 
    if (Elem[k].z<-S.hside) Elem[k].z+=S.side;
    return;
  }
  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[k].x=oldX;
      Elem[k].y=oldY;
      Elem[k].z=oldZ;
      Elem[k].flag=oldflag;
      Elem[k].adsflag=oldadsflag;
      Elem[k].chapflag = oldchapflag;

      if (Elem[k].x>=S.hside) Elem[k].x-=S.side;
      if (Elem[k].y>=S.hside) Elem[k].y-=S.side;
      if (Elem[k].z>=S.hside) Elem[k].z-=S.side;
      if (Elem[k].x<-S.hside) Elem[k].x+=S.side;  
      if (Elem[k].y<-S.hside) Elem[k].y+=S.side; 
      if (Elem[k].z<-S.hside) Elem[k].z+=S.side;
//set_Verlet();
      return;
      }
    }
  xyz_Ns:
  if (Elem[k].type==0){
    Elem[k].flag=0;
    if ((fabs(Elem[k].y)>=6.0) || (fabs(Elem[k].z)>=6.0)) Elem[k].flag=1;

    if (oldflag!=Elem[k].flag) {

      if ((oldflag==0) && (Elem[k].flag==1)){
	Ns=Ns+1;
	Nslist[Ns]=k;
	Nspos[k]=Ns;
	}
      if ((oldflag==1) && (Elem[k].flag==0)){
	Ns=Ns-1;
	for (j=Nspos[k];j<=Ns;j++) {
	  Nslist[j]=Nslist[j+1];
	  Nspos[Nslist[j+1]]=j;
	}
      }
    }
  }
  deltaEnergy = en_new - en_old;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;
  return;
}
	/*-----------------------------------------------------*/
void MC_rot(int k){

  double oldeX,oldeY,oldeZ,oldpX,oldpY,oldpZ;
  double dum1;
  double en_old,en_new;
  int oldadsflag,j, oldchapflag;
  vec3D dum2;
  double deltaEnergy = 0.;

  //if ((Elem[k].type == 3) || (Elem[k].type == 9)) return;
  if (Elem[k].type == 3) return;

  oldeX=Elem[k].ex;
  oldeY=Elem[k].ey;
  oldeZ=Elem[k].ez;
  oldpX=Elem[k].px;
  oldpY=Elem[k].py;
  oldpZ=Elem[k].pz;

  en_old=Interaction_1p(k);
  oldadsflag=Elem[k].adsflag;
  oldchapflag = Elem[k].chapflag;

  dum1=2*((0.5-drand48()));
  dum2=axisRND();

  Rotate(k,dum1,dum2);

  en_new=Interaction_1p(k);

  if ((int)(en_new)==999) {
    Elem[k].ex=oldeX;
    Elem[k].ey=oldeY;
    Elem[k].ez=oldeZ; 
    Elem[k].px=oldpX;
    Elem[k].py=oldpY;
    Elem[k].pz=oldpZ; 
    Elem[k].adsflag=oldadsflag;
    Elem[k].chapflag = oldchapflag;
    return;
  }
  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[k].ex=oldeX;
      Elem[k].ey=oldeY;
      Elem[k].ez=oldeZ;
      Elem[k].px=oldpX;
      Elem[k].py=oldpY;
      Elem[k].pz=oldpZ;
      Elem[k].adsflag=oldadsflag;
    Elem[k].chapflag = oldchapflag;
      return;
    }
  }
  deltaEnergy = en_new - en_old;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;

  if ((Elem[k].type==2) || (Elem[k].type==3)) MC_rot_p(k);
  //printf("patch %lf\n",en_new);
}
/*-----------------------------------------------------*/

void MC_rot_p(int k){

  double oldpX,oldpY,oldpZ;
  double en_old,en_new;
  double deltaEnergy = 0.;

  if (Elem[k].type == 3) return;

  oldpX=Elem[k].px;
  oldpY=Elem[k].py;
  oldpZ=Elem[k].pz;

  en_old=Interaction_1p(k);

  Rotate_p2(k,3*((0.5-drand48())));

  en_new=Interaction_1p(k);

  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[k].px=oldpX;
      Elem[k].py=oldpY;
      Elem[k].pz=oldpZ;
      return;
    }
  }
  //printf("patch %lf\n",en_new);
  deltaEnergy = en_new - en_old;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;

  return;
}
/**********************************************************
		 PISANJE / BRANJE DATOTEK
**********************************************************/
/*-----------------------------------------------*/
void read_Elements(void){
	   
  int i,number1,npart,type,frame0, ads;
  FILE *Aread, *Aread1;
  char to[200], to1[200];
  double number2, number3;
  int check1,check2;

  if (rate_simulation){
    sprintf(to,"../config.dat");
    sprintf(to1,"../track3");
    printf("%s %d \n",to, N0);
    Aread=fopen(to,"r");
    Aread1 = fopen(to1,"r");

    while(!feof(Aread1)){
      fscanf(Aread1, "%d %lf %lf %lf\n", &number1,&number2,&S.side,&number3);
    }  
    printf("\n S.side = %lf \n\n",S.side);
    S.hside=0.5*S.side;
    S.Volume=S.side*S.side*S.side;
    S.Volume0=S.side*S.side*S.side - S.side*12.0*12.0;
    fscanf(Aread, "%d %d\n", &npart,&frame0);
    N = npart;
    NEX = npart - N0 - Nchap;
    printf("S.side %lf movieframe %d N %d %d NEX %d\n", S.side,frame0,N,npart,NEX);
    Ns = 0;
    
    Elem=(PARTICLE *) malloc((100*N+1)*sizeof(PARTICLE));

    for (i=1;i<=N;i++){

      check1=fscanf(Aread,"%d %lf %lf %lf %lf %lf %lf\n",&Elem[i].type,&Elem[i].x,&Elem[i].y,&Elem[i].z,
          				   &Elem[i].ex,&Elem[i].ey,&Elem[i].ez);
      check2=fscanf(Aread,"%lf %lf %lf %d\n",&Elem[i].px,&Elem[i].py,&Elem[i].pz,&Elem[i].adsflag);
      assert(check1 == 7);
      assert(check2 == 4);
      if (Elem[i].type==0) Elem[i].eng=eng0;
      if (Elem[i].type==1) Elem[i].eng=deltaMu_ivbeta;
      if (Elem[i].type==2) Elem[i].eng=eng2;
      if (Elem[i].type==3) Elem[i].eng=0;
      if (Elem[i].type==4) Elem[i].eng=0;
      Elem[i].flag=0;
      if ((Elem[i].type == 9) && (Elem[i].adsflag == 2)) Elem[i].chapflag = 1;
      else Elem[i].chapflag = 0;
    }
    fclose(Aread);  
    fclose(Aread1);
  }
  else if (equilibrium_coverage_simulation){
    sprintf(to,"../seed1_%d.dat",N0);
    printf("%s %d \n",to, N0);
    Aread=fopen(to,"r");
    for (i=1;i<=N0;i++){

      check1=fscanf(Aread,"%d %lf %lf %lf %lf %lf %lf\n",&Elem[i].type,&Elem[i].x,&Elem[i].y,&Elem[i].z,
                                             &Elem[i].ex,&Elem[i].ey,&Elem[i].ez);
      check2=fscanf(Aread,"%lf %lf %lf\n",&Elem[i].px,&Elem[i].py,&Elem[i].pz);
      assert(check1 == 7);
      assert(check2 == 3);
      if (Elem[i].type==0) Elem[i].eng=eng0;
      if (Elem[i].type==1) Elem[i].eng=deltaMu_ivbeta;
      if (Elem[i].type==2) Elem[i].eng=eng2;
      if (Elem[i].type==3) Elem[i].eng=0;
      if (Elem[i].type==4) Elem[i].eng=0;
      Elem[i].flag=0;
      Elem[i].adsflag=0;
      Elem[i].chapflag = 0;
    }
    fclose(Aread);  
  }
}
/*-----------------------------------------------*/
int write_track4(int frejm){
  int count0, count1, count2, count3, countall, countChap, count_ads, count_chapads;
  int ii;
  count0=0; 
  count1=0; 
  count2=0; 
  count3=0; 
  countChap = 0; 
  count_ads = 0;
  count_chapads = 0;
  out=fopen("track4","a");
  for (ii=1;ii<=N;ii++){
    if (Elem[ii].type==0) count0++;
    if (Elem[ii].type==1) count1++;
    if (Elem[ii].type==2) count2++;
    if (Elem[ii].type==3) count3++;
    if (Elem[ii].type==9) countChap++;
    if ((Elem[ii].adsflag == 1) && ( (Elem[ii].type == 0) || (Elem[ii].type == 1) ) ) count_ads++;
    if ((Elem[ii].chapflag == 1) && (Elem[ii].type == 9)) count_chapads++;               
    }
  countall=count0+count1+count2+count3+countChap;
  fprintf(out,"%d %d %d %d %d %d %d %d %d \n",(int)((double)(frejm)/(double)(WRITE)),
					      count0,count1,count2,count3,countChap,
          				      countall, count_ads, count_chapads);
  fclose(out);
   return count2;
}
/*-----------------------------------------------*/
void write_exchanges(int frejm, int particle_identity){
  exchangefile = fopen("exchanges.dat","a");
  fprintf(exchangefile, "%d %d\n", frejm, particle_identity);
  fclose(exchangefile);
}
/*-----------------------------------------------*/
void makemovie(int movieframe){
  int i,adsflag;
  double x,y,z,x1,y1,z1;

  moviefile=fopen("moviewall33","a");
  fprintf(moviefile,"%d %d\n",N,movieframe);		// dodan WRITE, da niso prevelike cifre...
  for (i=1;i<=N;i++){
    x=Elem[i].x;
    y=Elem[i].y;
    z=Elem[i].z;
    x1=Elem[i].ex;
    y1=Elem[i].ey;
    z1=Elem[i].ez;
    fprintf(moviefile,"%d %lf %lf %lf %lf %lf %lf\n",Elem[i].type,x,y,z,x1,y1,z1);
    x=Elem[i].px;
    y=Elem[i].py;
    z=Elem[i].pz;
    adsflag = Elem[i].adsflag;		
    fprintf(moviefile,"%lf %lf %lf %d\n",x,y,z, adsflag);
  }
  fclose(moviefile);
}
void event(int eventframe){
  int count0, count1, count2, count3, countall, countChap, count_ads, count_chapads;
  int ii;
  int i,adsflag;
  double x,y,z,x1,y1,z1;

  eventfile=fopen("eventmovie","a");
  fprintf(eventfile,"%d %d\n",N,eventframe);
  for (i=1;i<=N;i++){
    x=Elem[i].x;
    y=Elem[i].y;
    z=Elem[i].z;
    x1=Elem[i].ex;
    y1=Elem[i].ey;
    z1=Elem[i].ez;
    fprintf(eventfile,"%d %lf %lf %lf %lf %lf %lf\n",Elem[i].type,x,y,z,x1,y1,z1);
    x=Elem[i].px;
    y=Elem[i].py;
    z=Elem[i].pz;
    adsflag = Elem[i].adsflag;		
    fprintf(eventfile,"%lf %lf %lf %d\n",x,y,z, adsflag);
  }
  fclose(eventfile);
  eventtrackfile=fopen("eventtrack","a");
  count0=0; 
  count1=0; 
  count2=0; 
  count3=0; 
  countChap = 0; 
  count_ads = 0;
  count_chapads = 0;
  for (ii=1;ii<=N;ii++){
    if (Elem[ii].type==0) count0++;
    if (Elem[ii].type==1) count1++;
    if (Elem[ii].type==2) count2++;
    if (Elem[ii].type==3) count3++;
    if (Elem[ii].type==9) countChap++;
    if ((Elem[ii].adsflag == 1) && ( (Elem[ii].type == 0) || (Elem[ii].type == 1) ) ) count_ads++;
    if ((Elem[ii].chapflag == 1) && (Elem[ii].type == 9)) count_chapads++;               
    }
  countall=count0+count1+count2+count3+countChap;
  fprintf(eventtrackfile,"%d %d %d %d %d %d %d %d %d \n",eventframe,
					      count0,count1,count2,count3,countChap,
          				      countall, count_ads, count_chapads);
  fclose(eventtrackfile);
}
/**********************************************************
	       SCALE, ADD, EXCHANGE, REMOVE
**********************************************************/

void Scale(void){

  int i,j,ch,k,la,check=0;
  bool devetdevet = false;
  int counter_rejectov = 0, counter_chap_rejectov = 0;
  double abs, theta, z, d2;

  for(k=N0+1;k<=N0+Nchap;k++){
    check = check_overlap(k);
    if ((fabs(Elem[k].x)>(S.hside-0.0001)) || 
        (fabs(Elem[k].y)>(S.hside-0.0001)) ||
        (fabs(Elem[k].z)>(S.hside-0.0001)) || (check == 1)){
      counter_chap_rejectov += 1;
      scale_chap_again:
 
      Elem[k].x=(.5-drand48())*(S.side);
      Elem[k].y=(.5-drand48())*(S.side);
      Elem[k].z=(.5-drand48())*(S.side);

      theta=drand48()*2*M_PI;
      z=(.5-drand48())*2;
      Elem[k].ex=sqrt(1.0-z*z)*cos(theta);
      Elem[k].ey=sqrt(1.0-z*z)*sin(theta);
      Elem[k].ez=z;

      for (j=1;j<=N;j++){
        if (j != k){
          d2=overlap(k,j,hlength,hlength);
          if (d2<S.sig){
//            printf("chaperone scale again    %lf  <   %lf za delca %d %d\n", d2, S.sig, k, j);      
            goto scale_chap_again;
          }
        }
      }
      Elem[k].px=(.5-drand48())*2;
      Elem[k].py=(.5-drand48())*2;
      Elem[k].pz=(-Elem[k].ex*Elem[k].px-Elem[k].ey*Elem[k].py)/Elem[k].ez;
      abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
      Elem[k].px=Elem[k].px/abs;
      Elem[k].py=Elem[k].py/abs;
      Elem[k].pz=Elem[k].pz/abs;
      abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
      Elem[k].chapflag = 0;
      Elem[k].adsflag = 0;
      Elem[k].flag = 0; 
      set_Verlet();
    }
  }
  for (i = N0+Nchap+1; i <= N; i++){
    //if (i == N0+Nchap+1){ printf("\n\nscale zacne s prvim elementom !!!!!!!!!!!\n\n"); devetdevet = true;}
    //if (!devetdevet) return;
    if ((fabs(Elem[i].x)>(S.hside-0.0001)) ||(fabs(Elem[i].y)>(S.hside-0.0001)) || (fabs(Elem[i].z)>(S.hside-0.0001))) {
	counter_rejectov += 1;
  //    printf("skaliranje||  halfside %lf | koordinate %lf %lf %lf \n",S.hside, Elem[i].x, Elem[i].y, Elem[i].z);
      for (j=i;j<=N-1;j++){
	Elem[j].type=Elem[j+1].type;
	Elem[j].x=Elem[j+1].x;
	Elem[j].y=Elem[j+1].y;
	Elem[j].z=Elem[j+1].z;
	Elem[j].ex=Elem[j+1].ex;
	Elem[j].ey=Elem[j+1].ey;
	Elem[j].ez=Elem[j+1].ez;
	Elem[j].px=Elem[j+1].px;
	Elem[j].py=Elem[j+1].py;
	Elem[j].pz=Elem[j+1].pz;
	Elem[j].eng=Elem[j+1].eng;
	Elem[j].flag=Elem[j+1].flag;
	Elem[j].adsflag=Elem[j+1].adsflag;
	Elem[j].chapflag=Elem[j+1].chapflag;
      }
      N=N-1;
      i = i-1;
    }
  }
  for (la=N0+Nchap+1;la<=N;la++){
    check=check_overlap(la);
    if (check==1) {
      printf("Delec %d izbrisan na koncu Scale() rutine\n", la);
      printf("Koordinate:   %lf   %lf    %lf  \n", Elem[la].x, Elem[la].y,Elem[la].z);
      for (j=la;j<=N-1;j++){
        Elem[j].type=Elem[j+1].type;
        Elem[j].x=Elem[j+1].x;
        Elem[j].y=Elem[j+1].y;
        Elem[j].z=Elem[j+1].z;
        Elem[j].ex=Elem[j+1].ex;
        Elem[j].ey=Elem[j+1].ey;
        Elem[j].ez=Elem[j+1].ez;
        Elem[j].px=Elem[j+1].px;
        Elem[j].py=Elem[j+1].py;
        Elem[j].pz=Elem[j+1].pz;
        Elem[j].eng=Elem[j+1].eng;
        Elem[j].flag=Elem[j+1].flag;
        Elem[j].adsflag=Elem[j+1].adsflag;
  	Elem[j].chapflag=Elem[j+1].chapflag;
      }
      erase_counter += 1;
      N=N-1;
      la = la-1;
    }
    check=0;
  }
  //preverim kako je s chaperoni
  for (ch=N0+1;ch<=N0+Nchap;ch++){
    if (fabs(Elem[ch].x) > (S.hside-0.0001)){
       printf("chaperone  %d  zunaj kocke \n", ch);
       chap_miss += 1;	}
    if (fabs(Elem[ch].y) > (S.hside-0.0001)){
       printf("chaperone  %d  zunaj kocke \n", ch);
       chap_miss += 1;	}
    if (fabs(Elem[ch].z) > (S.hside-0.0001)){
       printf("chaperone  %d  zunaj kocke \n", ch);
       chap_miss += 1;	}
  }
  printf("Med skaliranjem izbrisanih %d delcev in zamenjanih %d chapeorov\n", 
                                    counter_rejectov, counter_chap_rejectov);
  set_Verlet();
  devetdevet = false;
}
/********************************************************/
int check_overlap(int i){
  double d;
  int j;

  for (j=1;j<=N;j++){
    if (i!=j){
      d = overlap(i, j, hlength,hlength);
      if (d<S.sig) return 1; 
    }
  }
return 0;
}
/********************************************************/
/*-----------------------------*/
void Exchange1(int i){
  double d2,z,theta,flipacoin,abs,en_old,en_new;
  int k,j;
  double dum1,dum2;

  flipacoin=drand48();
  if (i<=N0+Nchap) printf("wrongly chosen\n");

  if (flipacoin<0.5) {
    en_old=Interaction_1p(i);
    if (en_old > 0.001) printf("\n\nError neki>> en_old = %lf\n\n", en_old);

    if (drand48()>((double)(N-N0-Nchap)*lambda3/S.Volume0)*exp(-(mu+0.0-en_old))) {
      return;
    }
    erase_counter += 1;
    //erase:
    for (j=i;j<=N-1;j++){
      Elem[j].type=Elem[j+1].type;
      Elem[j].x=Elem[j+1].x;
      Elem[j].y=Elem[j+1].y;
      Elem[j].z=Elem[j+1].z;
      Elem[j].ex=Elem[j+1].ex;
      Elem[j].ey=Elem[j+1].ey;
      Elem[j].ez=Elem[j+1].ez;
      Elem[j].px=Elem[j+1].px;
      Elem[j].py=Elem[j+1].py;
      Elem[j].pz=Elem[j+1].pz;
      Elem[j].eng=Elem[j+1].eng;
      Elem[j].flag=Elem[j+1].flag;
      Elem[j].adsflag=Elem[j+1].adsflag;
      Elem[j].chapflag=Elem[j+1].chapflag;
    }
    N=N-1;
    set_Verlet();
  }
  else if (flipacoin>=0.5) {
    //insert
    k=N+1;
    flipacoin=drand48();
    Elem[k].x=(.5-drand48())*(S.side);
    if (flipacoin>=0.5){
      dum1 = 6.01 + (S.hside - 6.0)*(drand48());
      dum2 = 0.5 - drand48();
      Elem[k].y=copysign(dum1, dum2);
      Elem[k].z=(.5-drand48())*(S.side);
    }
    if (flipacoin<0.5){
      Elem[k].y=(.5-drand48())*(S.side);
      dum1 = 6.01 + (S.hside - 6.0)*(drand48());
      dum2 = 0.5 - drand48();
      Elem[k].z=copysign(dum1, dum2);
    }
    theta=drand48()*2*M_PI;
    z=(.5-drand48())*2;
    Elem[k].ex=sqrt(1.0-z*z)*cos(theta);
    Elem[k].ey=sqrt(1.0-z*z)*sin(theta);
    Elem[k].ez=z;
    if ((fabs(Elem[k].y)<=6.0) && (fabs(Elem[k].z)<=6.0)) printf("wrong insert\n");
    for (j=1;j<=N;j++){
      d2=overlap(k,j,hlength,hlength);
      if (d2<S.sig) return;
    }
    Elem[k].px=(.5-drand48())*2;
    Elem[k].py=(.5-drand48())*2;
    Elem[k].pz=(-Elem[k].ex*Elem[k].px-Elem[k].ey*Elem[k].py)/Elem[k].ez;
    abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
    Elem[k].px=Elem[k].px/abs;
    Elem[k].py=Elem[k].py/abs;
    Elem[k].pz=Elem[k].pz/abs;
    Elem[k].type=0;
    Elem[k].flag=1;

    en_new=Interaction_1p2(k);

    if ((int)(en_new)==999) { // reject
      //  printf("no\n");
      return;
    }
    if (drand48()>(S.Volume0/(lambda3*(N-N0-Nchap+1)))*exp(mu-(en_new-0.0))) {
      return;
    }
    N=N+1;
    add_counter += 1;
    int count2_ads = 0;
    int ii;
    set_Verlet();
  }
  return;
}
/*-----------------------------*/
	/********************************************************/
void Exchange(int frejm, int ii){

  double d2,z,theta,flipacoin,abs,en_old=0.0,en_new=0.0;
  int i,k,j;
  double deltaEnergy = 0.;
  double dum1, dum2;
  exchange_counter += 1;

  i=Nslist[ii];
  flipacoin=drand48();
  if ((flipacoin<0.5) && (i != N)) {
    //check Metropolis first
    if ((fabs(Elem[i].y)<6.0) && (fabs(Elem[i].z)<6.0)) printf("something's wrong %d %d %d\n",frame,i,Elem[i].flag);

    en_old=Interaction_1p(i);
    en_new = 0.;

    if (drand48()>(((double)(Ns)*lambda3/S.Volume0)*exp(-(mu+0.0-en_old)))) {
      return;	// ne izbrisemo
    }
    //erase:
    for (j=i;j<=N-1;j++){
      Elem[j].type=Elem[j+1].type;
      Elem[j].x=Elem[j+1].x;
      Elem[j].y=Elem[j+1].y;
      Elem[j].z=Elem[j+1].z;
      Elem[j].ex=Elem[j+1].ex;
      Elem[j].ey=Elem[j+1].ey;
      Elem[j].ez=Elem[j+1].ez;
      Elem[j].px=Elem[j+1].px;
      Elem[j].py=Elem[j+1].py;
      Elem[j].pz=Elem[j+1].pz;
      Elem[j].eng=Elem[j+1].eng;
      Elem[j].flag=Elem[j+1].flag;
      Elem[j].adsflag=Elem[j+1].adsflag;
      Elem[j].chapflag=Elem[j+1].chapflag;
    }
    N=N-1;
    Ns=Ns-1;

    for (j=1;j<ii;j++) {
      if (Nslist[j]>i) {
	Nslist[j]=Nslist[j]-1;
	Nspos[Nslist[j]]=j;
      }
    }
    for (j=ii;j<=Ns;j++) {
      if (Nslist[j+1]>i) Nslist[j+1]=Nslist[j+1]-1;
      Nslist[j]=Nslist[j+1];
      Nspos[Nslist[j+1]]=j;
    }
    erase_counter += 1;	    
    set_Verlet();
    if ((rate_simulation) && !(equilibrium_coverage_simulation)){
      write_exchanges(frejm, -i);
    }
  }
  else if (flipacoin>=0.5) {
    //insert
    en_old = 0.;
    k=N+1;
    flipacoin=drand48();
    Elem[k].x=(.5-drand48())*(S.side);

    if (flipacoin>=0.5){
      dum1 = 6.01 + (S.hside - 6.0)*(drand48());
      dum2 = 0.5 - drand48();
      Elem[k].y=copysign(dum1, dum2);
      Elem[k].z=(.5-drand48())*(S.side);
    }
    else if (flipacoin<0.5){
      Elem[k].y=(.5-drand48())*(S.side);
      dum1 = 6.01 + (S.hside - 6.0)*(drand48());
      dum2 = 0.5 - drand48();
      Elem[k].z=copysign(dum1, dum2);
    }
    theta=drand48()*2*M_PI;
    z=(.5-drand48())*2;
    Elem[k].ex=sqrt(1.0-z*z)*cos(theta);
    Elem[k].ey=sqrt(1.0-z*z)*sin(theta);
    Elem[k].ez=z;
    if ((fabs(Elem[k].y)<=6.0) && (fabs(Elem[k].z)<=6.0)) printf("wrong insert\n");
    for (j=1;j<=N;j++){
      d2=overlap(k,j,hlength,hlength);
      if (d2<S.sig){
	return;
      }
    }
    Elem[k].px=(.5-drand48())*2;
    Elem[k].py=(.5-drand48())*2;
    Elem[k].pz=(-Elem[k].ex*Elem[k].px-Elem[k].ey*Elem[k].py)/Elem[k].ez;
    abs=sqrt(Elem[k].px*Elem[k].px+Elem[k].py*Elem[k].py+Elem[k].pz*Elem[k].pz);
    Elem[k].px=Elem[k].px/abs;
    Elem[k].py=Elem[k].py/abs;
    Elem[k].pz=Elem[k].pz/abs;
    Elem[k].type=0;
    Elem[k].flag=1;
    en_new=Interaction_1p2(k);

    if ((int)(en_new)==999) { // reject
    //  printf("no\n");
      printf("\n\nzavrni insert ne sme do tega prit, ker zavrnejo ze zgornje rutine\n\n");
      return;
    }

    if (drand48()>((S.Volume0/(lambda3*(Ns+1)))*exp(mu-(en_new-0.0)))) {
      return;
    }
    N=N+1;
    Ns=Ns+1;
    Nslist[Ns]=k;
    Nspos[k]=Ns;

    add_counter += 1;
    set_Verlet();
    if ((rate_simulation) && !(equilibrium_coverage_simulation)){
      write_exchanges(frejm, N);
    }
  }
  if (!skaliranje){
    deltaEnergy = en_new - en_old;
    deltaE_cumulative_over_WRITE += deltaEnergy;
    deltaE_cumulative_over_simulation += deltaEnergy;
  }
  return;
}
/**********************************************************
			   SWAP
**********************************************************/

void swapto1(int i, int frejm){

  double olde,en_old,en_new;
  int j,Ns_old,oldtype,oldflag, oldadsflag;
  double oldE_config, newE_config;
  double deltaEnergy = 0.;

  if (Elem[i].type==0) Elem[i].eng=eng0;
  if (Elem[i].type==2) Elem[i].eng=eng2;

  olde=Elem[i].eng;
  oldtype=Elem[i].type;
  oldE_config = Interaction_1p(i);
  en_old = oldE_config + Elem[i].eng;
  oldflag=Elem[i].flag;
  Ns_old=Ns;
  oldadsflag = Elem[i].adsflag;
  if (oldtype==0){
    Elem[i].type=1;
    Elem[i].eng=eng1;
    Elem[i].flag=0;
  }
  if (oldtype==2) {
    Elem[i].type=1;
    Elem[i].eng=deltaMu_ivbeta;
  }
  newE_config = Interaction_1p(i);
  en_new = newE_config + Elem[i].eng;
  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[i].eng=olde;
      Elem[i].type=oldtype;
      Elem[i].flag=oldflag;
      Elem[i].adsflag = oldadsflag;
      Ns=Ns_old;
      return;
    }
  }
  if ((oldtype==0) && (oldflag==1)){
    Ns=Ns-1;
    for (j=Nspos[i];j<=Ns;j++) {
      Nslist[j]=Nslist[j+1];
      Nspos[Nslist[j+1]]=j;
    }
  }
  deltaEnergy = newE_config - oldE_config;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;

  event(frejm);

  return;
}
/********************************************************/
void swap10(int i, int frejm){

  double olde,en_old,en_new;
  int oldtype,Ns_old,oldflag,oldadsflag;
  double oldE_config, newE_config;
  double deltaEnergy = 0.;

  Elem[i].eng=eng1;
  olde=Elem[i].eng;
  oldtype=Elem[i].type;
  oldE_config = Interaction_1p(i);
  en_old = oldE_config + Elem[i].eng;
  oldflag=Elem[i].flag;
  Ns_old=Ns;
  oldadsflag = Elem[i].adsflag;
  Elem[i].type=0;
  Elem[i].eng=eng0;
  newE_config = Interaction_1p(i);
  en_new = newE_config + Elem[i].eng;
  //check these flags
  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[i].eng=olde;
      Elem[i].type=oldtype;
      Ns=Ns_old;
      Elem[i].flag=oldflag;
      Elem[i].adsflag = oldadsflag;
      return;
    }
  }
  if ((fabs(Elem[i].y)>=6.0) || (fabs(Elem[i].z)>=6.0)) {
    Elem[i].flag=1;
    Ns=Ns+1;
    Nslist[Ns]=i;
    Nspos[i]=Ns;
  }
  deltaEnergy = newE_config - oldE_config;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;
  event(frejm);
  return;
}
/********************************************************/
void swap12(int i, int frejm){

  double abs,olde,en_old,en_new;
  int oldtype,oldadsflag;
  double oldE_config, newE_config;
  double deltaEnergy = 0.;

  Elem[i].eng=deltaMu_ivbeta;
  olde=Elem[i].eng;
  oldtype=Elem[i].type;
  oldadsflag = Elem[i].adsflag;
  oldE_config = Interaction_1p(i);
  en_old = oldE_config + Elem[i].eng;

  Elem[i].type=2;
  Elem[i].eng=eng2;
  Elem[i].px=(.5-drand48())*2;
  Elem[i].py=(.5-drand48())*2;
  Elem[i].pz=(-Elem[i].ex*Elem[i].px-Elem[i].ey*Elem[i].py)/Elem[i].ez;
  abs=sqrt(Elem[i].px*Elem[i].px+Elem[i].py*Elem[i].py+Elem[i].pz*Elem[i].pz);
  Elem[i].px=Elem[i].px/abs;
  Elem[i].py=Elem[i].py/abs;
  Elem[i].pz=Elem[i].pz/abs;

  newE_config = Interaction_1p(i);
  en_new = newE_config + Elem[i].eng;

  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[i].eng=olde;
      Elem[i].type=oldtype;
      Elem[i].adsflag = oldadsflag;
      return;
    }
  }
  deltaEnergy = newE_config - oldE_config;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;
  event(frejm);
  return;
}
/********************************************************/
void swap02(int i, int frejm){

  double abs,olde,en_old,en_new;
  int j,oldtype,Ns_old,oldflag, oldadsflag;
  double oldE_config, newE_config;
  double deltaEnergy;

  if (Elem[i].type==0) Elem[i].eng=eng0;
  if (Elem[i].type==2) Elem[i].eng=eng2;

  olde=Elem[i].eng;
  oldtype=Elem[i].type;
  oldE_config = Interaction_1p(i);
  en_old = oldE_config + Elem[i].eng;
  oldflag=Elem[i].flag;
  Ns_old=Ns;
  oldadsflag = Elem[i].adsflag;
//  printf("na zacetku znotraj swap adsflag = %d\n", Elem[i].adsflag);
  if (oldtype==0){
    Elem[i].type=2;
    Elem[i].eng=eng2;
    Elem[i].px=(.5-drand48())*2;
    Elem[i].py=(.5-drand48())*2;
    Elem[i].pz=(-Elem[i].ex*Elem[i].px-Elem[i].ey*Elem[i].py)/Elem[i].ez;
    abs=sqrt(Elem[i].px*Elem[i].px+Elem[i].py*Elem[i].py+Elem[i].pz*Elem[i].pz);
    Elem[i].px=Elem[i].px/abs;
    Elem[i].py=Elem[i].py/abs;
    Elem[i].pz=Elem[i].pz/abs;
    Elem[i].flag=0;
  }
//  printf("po pxpypz  znotraj swap adsflag = %d\n", Elem[i].adsflag);
  if (oldtype==2) {
    Elem[i].type=0;
    Elem[i].eng=eng0;
  }
//  printf("pred _1p(i) znotraj swap adsflag = %d\n", Elem[i].adsflag);

  newE_config = Interaction_1p(i);
//  printf("pooo _1p(i) znotraj swap adsflag = %d\n", Elem[i].adsflag);
  en_new = newE_config + Elem[i].eng;
  if (en_new>en_old) {
    if (drand48()>exp(-(en_new-en_old) / temperature)) {
      Elem[i].eng=olde;
      Elem[i].type=oldtype;
      Elem[i].flag=oldflag;
      Elem[i].adsflag = oldadsflag;
      Ns=Ns_old;
//      printf("neuspesen swap>>> old: %d   new: %d  oldadsflag: %d adsflag: %d\n",
//                             oldtype, Elem[i].type, oldadsflag, Elem[i].adsflag);
      return;
    }
  }
  if ((oldtype==0) &&  (oldflag==1)) {
    Ns=Ns-1;
    for (j=Nspos[i];j<=Ns;j++) {
      Nslist[j]=Nslist[j+1];
      Nspos[Nslist[j+1]]=j;
    }
  }
  if (oldtype==2)  {
    if ((fabs(Elem[i].y)>=6.0) || (fabs(Elem[i].z)>=6.0)) {
      Elem[i].flag=1;
      Ns=Ns+1;
      Nslist[Ns]=i;
      Nspos[i]=Ns;
    }
  }
  deltaEnergy = newE_config - oldE_config;
  deltaE_cumulative_over_WRITE += deltaEnergy;
  deltaE_cumulative_over_simulation += deltaEnergy;
//  printf("uspesen swap::: oldtype %d with adsflag = %d   newtype %d with adsflag = %d \n",
//                                      oldtype, oldadsflag, Elem[i].type, Elem[i].adsflag);
  event(frejm);
  return;
}
/**********************************************************
                      INTERAKCIJE
**********************************************************/

double Interaction_1p(int k){

  double d,ee=0.;
  int j,dum,dumtype, ktype;
  Elem[k].adsflag=0;
  Elem[k].chapflag=0;
  ktype = Elem[k].type;

  for (j=1;j<=Elem[k].Nverl;j++){					

    dum = Elem[k].verl[j];
    dumtype = Elem[dum].type;    

    d = overlap(k, dum, hlength, hlength);
    if (d < S.sig) return 999.0;
    else if (d < S.cut){
      if (Elem[k].type==Elem[dum].type) ee+=sameatt(k,dum);
      if (Elem[k].type!=Elem[dum].type) ee+=diffatt(k,dum);
    }
  }
  return ee;
}
/********************************************************/
/********************************************************/
double Interaction_1p2(int k){

  int j,dum,dumtype;
  double d, ee=0.0;
  int ktype = Elem[k].type;

  for (j=1;j<=N;j++){					
    dum = j;
    dumtype = Elem[dum].type;    

    if (k != dum){

      d = overlap(k, dum, hlength, hlength);
      if (d < S.sig) return 999.0;
      else if (d < S.cut){
          if (Elem[k].type==Elem[dum].type) ee+=sameatt(k,dum);
          if (Elem[k].type!=Elem[dum].type) ee+=diffatt(k,dum);
      }
    }  
  }
  return ee;
}
/********************************************************/
double system_potential_energy(void){

  int i,j,itype,jtype;
  double d, ee=0.0, lolo, lele;

  for (i=1;i<=N;i++){					
    itype = Elem[i].type;
    for (j=i+1;j<=N;j++){
      jtype = Elem[j].type;    

      d = overlap(i, j, hlength, hlength);
      if (d < S.sig){
        lolo = Interaction_1p2(i);  
        lele = Interaction_1p(i);
        printf("system_po_en overlap %d %d %lf %lf\n", i, j, lolo, lele);
        printf(" %lf %lf %lf %lf %lf %lf \n %lf %lf %lf %lf %lf %lf \n",
 						 Elem[i].x, Elem[i].y, Elem[i].z,
 					         Elem[i].ex, Elem[i].ey, Elem[i].ey,
                                                 Elem[j].x, Elem[j].y, Elem[j].z,
						 Elem[j].ex, Elem[j].ey, Elem[j].ez);
        fak = 1;
      }
      else if (d < S.cut){
          if (Elem[i].type==Elem[j].type) ee+=sameatt(i, j);
          if (Elem[i].type!=Elem[j].type) ee+=diffatt(i, j);
      }
    }    
  }
return ee;
}
/********************************************************/
double system_potential_energy_inverse(void){

  int i,j, itype,jtype;
  double d, ee=0.0, lolo, lele;

  for (i=N;i>=1;i--){					
    itype = Elem[i].type;
  
    for (j=i-1;j>=1;j--){
      jtype = Elem[j].type;    

      d = overlap(i, j, hlength, hlength);
      if (d < S.sig){
        lolo = Interaction_1p2(i);  
        lele = Interaction_1p(i);
        printf("system_po_en overlap %d %d %lf %lf\n", i, j, lolo, lele);
        fak = 1;
      }
      else if (d < S.cut){
          if (Elem[i].type==Elem[j].type) ee+=sameatt(i, j);
          if (Elem[i].type!=Elem[j].type) ee+=diffatt(i, j);
      }
    }    
  }
return ee;
}
/********************************************************/
void calculate_binding_energies(int i){

int countads, countchapads, nn, ff, ss, fff;
double short_energy, wall_energy, micelle_energy;
double energija_micelle, energija_short, energija_wall, delta_short, delta_wall, delta_micelle;
double chap_vezi_na_fibril, oligomer_bound_proportion, sol_vezi_na_fibril;
int oligomer_bound, sol_to_fibril, chap_to_fibril;
int delitelj;

  kaunt += 1;
  countads = 0;
  countchapads = 0;
  short_energy = 0.0;
  wall_energy = 0.0;
  micelle_energy = 0.0;
  oligomer_bound = 0;
  chap_to_fibril = 0;
  sol_to_fibril = 0;

  for (nn=1;nn<=N;nn++){
    if ((Elem[nn].type == 0)	&& (Elem[nn].adsflag == 1)){
      countads += 1;
      for (fff=1;fff<=N0;fff++){
        energija_wall = wallattraction(nn,fff,epsilonwall0);
        if (energija_wall < -0.0001) sol_to_fibril += 1;
        wall_energy +=  energija_wall;
      }
      for (ss=N0+Nchap+1;ss<=N;ss++){
        if ((Elem[ss].adsflag == 1) && (ss != nn) && (Elem[ss].type == 0)){
          energija_micelle = micelle(nn,ss,epsilonmic0);
          if (energija_micelle < -0.0001) oligomer_bound += 1;
          micelle_energy += energija_micelle;
        }
    //    if (Elem[ss].type != 0) printf("%d", Elem[ss].type);
      }
    }
    if ((Elem[nn].type == 9) && (Elem[nn].chapflag == 1)){
      countchapads += 1;
      for (ff=1;ff<=N0;ff++){
        energija_short = wallattraction(nn, ff, epsilon_chaperone_fibril);
        if (energija_short < -0.0001) chap_to_fibril += 1; // presteje za posamezen chaperone s kolikimi f se veze
        short_energy += energija_short;
      }      
    }
  }
  assert((oligomer_bound % 2) == 0);
  if (countchapads >= 1) short_energy = short_energy / (double)(countchapads);
  else short_energy = 0.0;
  if (countads >= 1)  wall_energy = wall_energy / (double)(countads);
  else wall_energy = 0.0;
  if ((oligomer_bound >= 1) && (countads >= 1)) micelle_energy = micelle_energy / (double)(oligomer_bound);
  else micelle_energy = 0.0;
  // Izracun variance in povprecja...
  if ((kaunt % kaunt_freq) != 0) delitelj = kaunt % kaunt_freq;
  else delitelj = kaunt_freq;

     delta_short = short_energy - short_average;
     short_average += delta_short / (double)(delitelj);
     short_variance = short_variance + (delta_short)*(short_energy - short_average); 

     delta_wall = wall_energy - wall_average;
     wall_average += delta_wall / (double)(delitelj);
     wall_variance = wall_variance + (delta_wall)*(wall_energy - wall_average); 

     delta_micelle = micelle_energy - micelle_average;
     micelle_average += delta_micelle / (double)(delitelj);
     micelle_variance = micelle_variance + (delta_micelle)*(micelle_energy - micelle_average); 


  if (kaunt % kaunt_freq == 0){
  
    if (countchapads >= 1) chap_vezi_na_fibril = (double)(chap_to_fibril) / (double)(countchapads);
    else chap_vezi_na_fibril = 0.0;
    short_variance = sqrt(short_variance / (double)(kaunt_freq - 1));
    if (countads >= 1) sol_vezi_na_fibril = (double)(sol_to_fibril) / (double)(countads);
    else sol_vezi_na_fibril = 0.0;
    wall_variance = sqrt(wall_variance / (double)(kaunt_freq - 1));
    oligomer_bound = oligomer_bound / 2;
    if (countads >= 1) oligomer_bound_proportion = (double)(oligomer_bound) / (double)(countads);
    else oligomer_bound_proportion = 0.0;
    micelle_variance = sqrt(micelle_variance / (double)(kaunt_freq - 1));

   bindingfile = fopen("binding_energies.dat","a");
   fprintf(bindingfile,"%d %lf %lf %d %d %lf %lf %lf %d %d %lf %lf %lf %d %d %lf\n",
           i, short_average, short_variance, countchapads, chap_to_fibril, chap_vezi_na_fibril,
           wall_average, wall_variance, countads, sol_to_fibril, sol_vezi_na_fibril, 
           micelle_average, micelle_variance, countads, oligomer_bound, oligomer_bound_proportion);   
   fclose(bindingfile);
   short_average = 0.0;
   short_variance = 0.0;
   wall_average = 0.0;
   wall_variance = 0.0;
   micelle_average = 0.0;
   micelle_variance = 0.0; 
  }
}
/********************************************************/
double sameatt(int i,int j){
  double att=0.0;
  if (Elem[i].type==0) att=micelle(i,j,epsilonmic0);
  if (Elem[i].type==1) att=micelle(i,j,epsilonmic1);
  if (Elem[i].type==2) att=patch(i,j);
 // if (Elem[i].type==3) att=patch(i,j);
  return att;
}
/********************************************************/
double diffatt(int i,int j){
  double att=0.0;

  if ((Elem[i].type==0) && (Elem[j].type==2)) att=mixedatt(i,j,epsilonmix0);
  if ((Elem[i].type==2) && (Elem[j].type==0)) att=mixedatt(j,i,epsilonmix0);

  if ((Elem[i].type==1) && (Elem[j].type==2)) att=mixedatt(i,j,epsilonmix1);
  if ((Elem[i].type==2) && (Elem[j].type==1)) att=mixedatt(j,i,epsilonmix1);

  if ((Elem[i].type==0) && (Elem[j].type==1)) att=micelle(i,j,epsilonmic01);
  if ((Elem[i].type==1) && (Elem[j].type==0)) att=micelle(j,i,epsilonmic01);

  if ((Elem[i].type==0) && (Elem[j].type==3)) att=wallattraction(i,j,epsilonwall0);
  if ((Elem[i].type==3) && (Elem[j].type==0)) att=wallattraction(j,i,epsilonwall0);
  if ((Elem[i].type==1) && (Elem[j].type==3)) att=wallattraction(i,j,epsilonwall1);
  if ((Elem[i].type==3) && (Elem[j].type==1)) att=wallattraction(j,i,epsilonwall1);

  if ((Elem[i].type == 9) && (Elem[j].type == 3)) att = wallattraction(i, j, epsilon_chaperone_fibril);
  if ((Elem[i].type == 3) && (Elem[j].type == 9)) att = wallattraction(j, i, epsilon_chaperone_fibril);

  return att;
}
/********************************************************/
double micelle(int i,int j, double epsilon) {

  double endx,endy,endz,endx1,endy1,endz1,endx2,endy2,endz2,dend,dum,ee=0.0;

  endx1=Elem[i].x+hlength*Elem[i].ex;
  endy1=Elem[i].y+hlength*Elem[i].ey;
  endz1=Elem[i].z+hlength*Elem[i].ez;
  if (endx1>=S.hside) endx1-=S.side;
  if (endy1>=S.hside) endy1-=S.side;
  if (endz1>=S.hside) endz1-=S.side;

  if (endx1<-S.hside) endx1+=S.side;
  if (endy1<-S.hside) endy1+=S.side;
  if (endz1<-S.hside) endz1+=S.side;

  endx2=Elem[j].x+hlength*Elem[j].ex;
  endy2=Elem[j].y+hlength*Elem[j].ey;
  endz2=Elem[j].z+hlength*Elem[j].ez;
  if (endx2>=S.hside) endx2-=S.side;
  if (endy2>=S.hside) endy2-=S.side;
  if (endz2>=S.hside) endz2-=S.side;

  if (endx2<-S.hside) endx2+=S.side;
  if (endy2<-S.hside) endy2+=S.side;
  if (endz2<-S.hside) endz2+=S.side;

  endx=endx1-endx2;
  endy=endy1-endy2;
  endz=endz1-endz2;
  if (endx>=S.hside) endx-=S.side;
  if (endy>=S.hside) endy-=S.side;
  if (endz>=S.hside) endz-=S.side;

  if (endx<-S.hside) endx+=S.side;
  if (endy<-S.hside) endy+=S.side;
  if (endz<-S.hside) endz+=S.side;

  dend=sqrt(endx*endx+endy*endy+endz*endz);
  if (dend < S.cut){
    dum=(S.sig/dend)*(S.sig/dend);
    dum=dum*dum*dum;
    ee=-epsilon*dum;
  }
  return ee;
}
/********************************************************/
double mixedatt(int i,int j, double epsilon){

  double ex1,ey1,ez1,rij,rijx,rijy,rijz,rij2,att=0.0;
  double x3;
  double absB,projBx,projBy,projBz,cr2,d;

  d=overlap(i,j,htlength,hblength);

  if (d<S.cut) {
    ex1=Elem[i].x+(hlength-htlength)*Elem[i].ex;
    ey1=Elem[i].y+(hlength-htlength)*Elem[i].ey;
    ez1=Elem[i].z+(hlength-htlength)*Elem[i].ez;

    if (ex1>=S.hside) ex1-=S.side;
    if (ey1>=S.hside) ey1-=S.side;
    if (ez1>=S.hside) ez1-=S.side;

    if (ex1<-S.hside) ex1+=S.side;
    if (ey1<-S.hside) ey1+=S.side;
    if (ez1<-S.hside) ez1+=S.side;

    rijx=Elem[j].x-(ex1);
    rijy=Elem[j].y-(ey1);
    rijz=Elem[j].z-(ez1);

    if (rijx>=S.hside) rijx-=S.side;
    if (rijy>=S.hside) rijy-=S.side;
    if (rijz>=S.hside) rijz-=S.side;

    if (rijx<-S.hside) rijx+=S.side;
    if (rijy<-S.hside) rijy+=S.side;
    if (rijz<-S.hside) rijz+=S.side;

    rij2=(rijx*rijx)+(rijy*rijy)+(rijz*rijz);
    rij=sqrt(rij2);


    x3=(rijx*Elem[j].ex+rijy*Elem[j].ey+rijz*Elem[j].ez)/rij;
    projBx=-rijx-x3*rij*Elem[j].ex;
    projBy=-rijy-x3*rij*Elem[j].ey;
    projBz=-rijz-x3*rij*Elem[j].ez;
    absB=sqrt(projBx*projBx+projBy*projBy+projBz*projBz);
    projBx=projBx/absB;
    projBy=projBy/absB;
    projBz=projBz/absB;

    cr2=projBx*Elem[j].px+projBy*Elem[j].py+projBz*Elem[j].pz;

    if (cr2>ang1){
      att=-epsilon;
      }
    }
  return att;
}
/********************************************************/
double wallattraction(int i,int j,double epsilon){

  double att=0.0, dum, d;

  d=overlap(i,j,htlength,hblength);
//  if (skaliranje)  printf("particles i = %d j = %d distance = %lf\n", i,j,d);
  if ((d<(S.cut)) && (d>S.sig)) {
//    if (skaliranje) printf("INSIDE particles i = %d j = %d distance = %lf\n", i,j,d);

    if ((Elem[i].type == 0) || (Elem[i].type == 1)){
      Elem[i].adsflag=1;
    }
    if (Elem[i].type == 9){
      Elem[i].adsflag=2;
      Elem[i].chapflag=1;
    }
    dum=(S.sig/d)*(S.sig/d);
    dum=dum*dum*dum;
    att=-epsilon*dum;
  }
  return att;

}
/********************************************************/
double patch(int i,int j){

  double rij,rijx,rijy,rijz,rij2,att;
  double c1x,c1y,c1z,c2x,c2y,c2z;
  double x1,x2,x3,P1,dum;
  double absA,absB,projAx,projAy,projAz,projBx,projBy,projBz,cr1,cr2;

  att=0.0;

  c1x=Elem[i].x;
  c1y=Elem[i].y;
  c1z=Elem[i].z;
  c2x=Elem[j].x;
  c2y=Elem[j].y;
  c2z=Elem[j].z;

  rijx=c2x-c1x;
  rijy=c2y-c1y;
  rijz=c2z-c1z;

  if (rijx>=S.hside) rijx-=S.side;
  if (rijy>=S.hside) rijy-=S.side;
  if (rijz>=S.hside) rijz-=S.side;
  if (rijx<-S.hside) rijx+=S.side;
  if (rijy<-S.hside) rijy+=S.side;
  if (rijz<-S.hside) rijz+=S.side;

  rij2=(rijx*rijx)+(rijy*rijy)+(rijz*rijz);
  rij=sqrt(rij2);
  if (rij<(blength)){

    x1=Elem[i].ex*Elem[j].ex+Elem[i].ey*Elem[j].ey+Elem[i].ez*Elem[j].ez;
    x2=(rijx*Elem[i].ex+rijy*Elem[i].ey+rijz*Elem[i].ez)/rij;
    x3=(rijx*Elem[j].ex+rijy*Elem[j].ey+rijz*Elem[j].ez)/rij;
    P1=x1*x1;
    //P2=1-x2*x2;
    //P3=1-x3*x3;
    projAx=rijx-x2*rij*Elem[i].ex;
    projAy=rijy-x2*rij*Elem[i].ey;
    projAz=rijz-x2*rij*Elem[i].ez;
    absA=sqrt(projAx*projAx+projAy*projAy+projAz*projAz);
    projAx=projAx/absA;
    projAy=projAy/absA;
    projAz=projAz/absA;

    projBx=-rijx-x3*rij*Elem[j].ex;
    projBy=-rijy-x3*rij*Elem[j].ey;
    projBz=-rijz-x3*rij*Elem[j].ez;
    absB=sqrt(projBx*projBx+projBy*projBy+projBz*projBz);
    projBx=projBx/absB;
    projBy=projBy/absB;
    projBz=projBz/absB;

    cr1=projAx*Elem[i].px+projAy*Elem[i].py+projAz*Elem[i].pz;
    cr2=projBx*Elem[j].px+projBy*Elem[j].py+projBz*Elem[j].pz;
    if ((cr1>ang1) && (cr2>ang1)){
      dum=(S.sig/rij);
      dum=dum*dum*dum;
      att=(-epsilonpatch1*P1-epsilonpatch2*dum);
    }
  }
  return att;
}
/*********************************************************************
                POMOŽNE ZADEVE - vektorske operacije, random, 
**********************************************************************/
void Rotate(int k,double an, vec3D n){

  double cn,sn;
  double w,x,y,z;
  double xx,yy,zz,xy,xz,yz,wx,wy,wz;
  vec3D v,p;
  double Rot[3+1][3+1];

  cn=cos(.5*an*M_PI/180.);
  sn=sin(.5*an*M_PI/180.);
  w=cn;
  x=n.x*sn;
  y=n.y*sn;
  z=n.z*sn;
  xx=x*x;
  yy=y*y;
  zz=z*z;
  xy=x*y;
  xz=x*z;
  yz=y*z;
  wx=w*x;
  wy=w*y;
  wz=w*z;
  Rot[1][1]=1.-2.*(yy+zz);
  Rot[2][1]=2.*(xy-wz);
  Rot[3][1]=2.*(xz+wy);
  Rot[1][2]=2.*(xy+wz);
  Rot[2][2]=1.-2.*(xx+zz);
  Rot[3][2]=2.*(yz-wx);
  Rot[1][3]=2.*(xz-wy);
  Rot[2][3]=2.*(yz+wx);
  Rot[3][3]=1.-2.*(xx+yy);
  v.x=Rot[1][1]*Elem[k].ex+Rot[2][1]*Elem[k].ey+Rot[3][1]*Elem[k].ez;
  v.y=Rot[1][2]*Elem[k].ex+Rot[2][2]*Elem[k].ey+Rot[3][2]*Elem[k].ez;
  v.z=Rot[1][3]*Elem[k].ex+Rot[2][3]*Elem[k].ey+Rot[3][3]*Elem[k].ez;
  p.x=Rot[1][1]*Elem[k].px+Rot[2][1]*Elem[k].py+Rot[3][1]*Elem[k].pz;
  p.y=Rot[1][2]*Elem[k].px+Rot[2][2]*Elem[k].py+Rot[3][2]*Elem[k].pz;
  p.z=Rot[1][3]*Elem[k].px+Rot[2][3]*Elem[k].py+Rot[3][3]*Elem[k].pz;
  Elem[k].px=p.x;
  Elem[k].py=p.y;
  Elem[k].pz=p.z;
  Elem[k].ex=v.x;
  Elem[k].ey=v.y;
  Elem[k].ez=v.z;
}
/*-----------------------------------------------*/
void Rotate_p2(int k,double an){

  double cn,sn;
  double w,x,y,z;
  double xx,yy,zz,xy,xz,yz,wx,wy,wz;
  vec3D v;
  double Rot[3+1][3+1];

  cn=cos(.5*an*M_PI/180.);
  sn=sin(.5*an*M_PI/180.);
  w=cn;
  x=Elem[k].ex*sn;
  y=Elem[k].ey*sn;
  z=Elem[k].ez*sn;
  xx=x*x;
  yy=y*y;
  zz=z*z;
  xy=x*y;
  xz=x*z;
  yz=y*z;
  wx=w*x;
  wy=w*y;
  wz=w*z;

  Rot[1][1]=1.-2.*(yy+zz);
  Rot[2][1]=2.*(xy-wz);
  Rot[3][1]=2.*(xz+wy);
  Rot[1][2]=2.*(xy+wz);
  Rot[2][2]=1.-2.*(xx+zz);
  Rot[3][2]=2.*(yz-wx);
  Rot[1][3]=2.*(xz-wy);
  Rot[2][3]=2.*(yz+wx);
  Rot[3][3]=1.-2.*(xx+yy);

  v.x=Rot[1][1]*Elem[k].px+Rot[2][1]*Elem[k].py+Rot[3][1]*Elem[k].pz;
  v.y=Rot[1][2]*Elem[k].px+Rot[2][2]*Elem[k].py+Rot[3][2]*Elem[k].pz;
  v.z=Rot[1][3]*Elem[k].px+Rot[2][3]*Elem[k].py+Rot[3][3]*Elem[k].pz;
  Elem[k].px=v.x;
  Elem[k].py=v.y;
  Elem[k].pz=v.z;
}
//-----------------------------------------------//
vec3D axisRND(void){
  vec3D r;
  double w, t;
  r.z = 2.0 * ran3(&Xseed) - 1.0;
  t = 2.0 * M_PI * ran3(&Xseed);
  w = sqrt( 1 - r.z*r.z );
  r.x = w * cos( t );
  r.y = w * sin( t );
  return r;
}
/****************************************************/
double VectorProductAbs(vec3D a ,vec3D b){
  vec3D c;
  double d;
  c.x=(a.y*b.z-a.z*b.y);
  c.y=(a.z*b.x-a.x*b.z);
  c.z=(a.x*b.y-a.y*b.x);
  d=sqrt(c.x*c.x+c.y*c.y+c.z*c.z);
  return d;
}
/****************************************************/
double ran3(long *idum){

  static int inext,inextp;
  static long ma[56];
  static int iff=0;
  long mj,mk;
  int i,ii,k;

  if (*idum < 0 || iff == 0) {
  iff=1;
  mj=MSEED-(*idum < 0 ? -*idum : *idum);
  mj %= MBIG;
  ma[55]=mj;
  mk=1;
  for (i=1;i<=54;i++) {
  ii=(21*i) % 55;
  ma[ii]=mk;
  mk=mj-mk;
  if (mk < MZ) mk += MBIG;
  mj=ma[ii];
  }
  for (k=1;k<=4;k++)
  for (i=1;i<=55;i++) {
  ma[i] -= ma[1+(i+30) % 55];
  if (ma[i] < MZ) ma[i] += MBIG;
  }
  inext=0;
  inextp=31;
  *idum=1;
  }
  if (++inext == 56) inext=1;
  if (++inextp == 56) inextp=1;
  mj=ma[inext]-ma[inextp];
  if (mj < MZ) mj += MBIG;
  ma[inext]=mj;
  return mj*FAC;
}
/**********************************************************
                      OVERLAP RUTINE
**********************************************************/
double overlap(int i,int j, double hl1, double hl2) {// Najkrajsa r azdalja med dvema sferocilindroma - namenjena rejectu, ne interakciji

  double x1,y1,z1,x2,y2,z2,x12,y12,z12,ex1,ey1,ez1,ex2,ey2,ez2;
  double e1r,e2r,e12,recipr,xla0,xmu0,xx12,yy12,zz12,rnsq,h1,h2,risq;
  double gamm,gam,gam1,gam2,gamms,del1,del2,del,delm,delms,aa,a1,a2,f1,f2,rclsq,rcl;

  if ((hl1 == hlength) && (hl2 == hlength)){
    if (Elem[i].type == 9) hl1 = hchap_length;	// ALLOW FOR DIFFERENT CHAPERONE SPC LENGTH
    if (Elem[j].type == 9) hl2 = hchap_length;
    x1=Elem[i].x; 
    y1=Elem[i].y;	
    z1=Elem[i].z;		
    x2=Elem[j].x; 
    y2=Elem[j].y;		
    z2=Elem[j].z;
    //TU JE GLAVNA RAZLIKA MED OVERLAP RUTINAMI
    h1=hl1; 
    h2=hl2;
  }
  else if ((hl1 == htlength) && (hl2 == hblength)){
    if (Elem[i].type == 9){
      hl1 = htchap_length;	// ALLOW FOR DIFFERENT chaperone tip length 
      x1=Elem[i].x + (hchap_length - htchap_length) * Elem[i].ex;
      y1=Elem[i].y + (hchap_length - htchap_length) * Elem[i].ey;
      z1=Elem[i].z + (hchap_length - htchap_length) * Elem[i].ez;
    }
    else{
      x1=Elem[i].x + (hlength - htlength) * Elem[i].ex;
      y1=Elem[i].y + (hlength - htlength) * Elem[i].ey;
      z1=Elem[i].z + (hlength - htlength) * Elem[i].ez;
    }
    if (x1>=S.hside) x1-=S.side;
    if (y1>=S.hside) y1-=S.side;
    if (z1>=S.hside) z1-=S.side;
    if (x1<-S.hside) x1+=S.side;
    if (y1<-S.hside) y1+=S.side;
    if (z1<-S.hside) z1+=S.side;
    x2=Elem[j].x; 
    y2=Elem[j].y;		
    z2=Elem[j].z;
    //TU JE GLAVNA RAZLIKA MED OVERLAP RUTINAMI
    h1=hl1; 
    h2=hl2;
  }
  else {
    printf("nesto narobe!!!!!!\n\n\n\n");
    fak = 1;
  }
  x12=x2-x1;
  y12=y2-y1;
  z12=z2-z1;
  if (x12>=S.hside) x12-=S.side;
  if (y12>=S.hside) y12-=S.side;
  if (z12>=S.hside) z12-=S.side;

  if (x12<-S.hside) x12+=S.side;
  if (y12<-S.hside) y12+=S.side;
  if (z12<-S.hside) z12+=S.side;

  ex1=Elem[i].ex; 
  ey1=Elem[i].ey;		
  ez1=Elem[i].ez;		
  ex2=Elem[j].ex;
  ey2=Elem[j].ey;
  ez2=Elem[j].ez;

  e1r=ex1*x12+ey1*y12+ez1*z12;
  e2r=ex2*x12+ey2*y12+ez2*z12;
  e12=ex1*ex2+ey1*ey2+ez1*ez2;

  if(fabs(1-e12*e12)<1e-4) {
    if((fabs(e1r)>1e-4) && (fabs(e2r)>1e-4)) {
      xla0= e1r*0.5;
      xla0= e1r*0.5;
      xmu0= -e2r*0.5;
      }
    else return sqrt(x12*x12+y12*y12+z12*z12);
    }

  else{  
    recipr=1./(1.-e12*e12);
    xla0=recipr*(e1r-e12*e2r);
    xmu0=-recipr*(e2r-e12*e1r);
    }
  xx12=x12+xmu0*ex2-xla0*ex1;
  yy12=y12+xmu0*ey2-xla0*ey1;
  zz12=z12+xmu0*ez2-xla0*ez1;
  rnsq=xx12*xx12+yy12*yy12+zz12*zz12;

  //-----------------------------------------

  if (((xla0*xla0)<=(h1*h1)) && ((xmu0*xmu0)<=(h2*h2))) {
    risq=0.0; 
    }
  else{
    gam1=-xla0-h1;
    gam2=-xla0+h1;
    gamm=gam1;
    if ((gam1*gam1) > (gam2*gam2)) gamm=gam2;
    del1=-xmu0-h2;
    del2=-xmu0+h2;
    delm=del1;
    if ((del1*del1) > (del2*del2)) delm=del2;	

    gam=gamm;	  
    delms=gam*e12;	  
    aa=xmu0+delms;
    if (aa*aa<=h2*h2) {
      del=delms;
      } 
    else{
      del=del1;
      a1=delms-del1;
      a2=delms-del2;
      if ((a1*a1)>(a2*a2)) del=del2;
      }
    f1=gam*gam+del*del-2.*gam*del*e12;

    del=delm;	  
    gamms=del*e12;	  
    aa=xla0+gamms;
    if ((aa*aa)<=(h1*h1)) {
      gam=gamms;
      }  
    else {
      gam=gam1;
      a1=gamms-gam1;
      a2=gamms-gam2;
      if ((a1*a1)>(a2*a2)) gam=gam2;
      }
    f2=gam*gam+del*del-2.*gam*del*e12;

    risq=f1;
    if (f2<f1) risq=f2;

    }
  rclsq=rnsq+risq;
  rcl=sqrt(rclsq);
  return rcl;
}
