##############################################################################
#
# This is an example job script for use on zero. This one is for jobs using
# the MPI-CH MPI library, ie the module mpi/mpich .
#
##############################################################################
#
# You can use lines starting #PBS to configure your job's parameters. These
# are read by the queueing system.
#

# The name of the job
#PBS -N eqsim040_eq4.50

    #PBS -o stdout_file
    #PBS -e stderr_file

# Queue to use: to see the full list of queues, type 'qstat -q'
#PBS -q l1

# Walltime limit (hours:mins:secs)
#PBS -l walltime=120:00:00

##############################################################################
#
# Now the actual script starts. Do not put PBS directives after this point.
# PBS uses your login shell to interpret the script- you cannot write it in
# csh if your login shell is bash.
#
##############################################################################

# Change to the directory you ran qsub from
cd ${PBS_O_WORKDIR}

# Write out some helpful info to the output file
echo "Starting job $PBS_JOBID"
echo
echo "PBS assigned me these nodes:"
cat $PBS_NODEFILE
echo

# PBS starts even a multi-processor job on a single node. It's up to the job to
# launch processes on other nodes; on zero this is handled by various
# launchers.  It can still be useful to know which node you started on, hence
# this bit:

echo First task running on:
echo `uname -a`
echo at `date`

# Start the job. mpiexec is a launcher program that talks to the queueing
# system and launches the correct number of processes in the correct places.
# It should be used for starting MPI-CH jobs. 

    ./Temp.exe | tee screen.txt

# Write out the PBS details of the job. If it ran for any sensible length
# of time this will tell you some useful statistics about it.
echo
echo "Job finished. PBS details are:"
echo
qstat -f ${PBS_JOBID}
echo
echo Finished at `date`

